<?php

/**
 * Global functions, placed here since they are needed several places
 *
 * @package Vikingboard
 * @author Tor Henning Ueland
 * @author Jørgen tellnes
 * @author Marius Pedersen Taule
 * @license GPL {@link http://www.gnu.org/licenses/gpl.html}
 *
 * $Id: functions.php 1867 2007-11-27 20:06:48Z jorgenboard $
 */

/**
 * Emulates MySQLs strip function, strips disturbing stuff
 * from strings going to SQL.
 *
 * @todo Not sure if this is as good as mysql's own escape string thingy,
 * 		we should rather use mysql's own stuff in the SQL drivers.
 *
 * @param string $string
 * @return string $string
 */
function sql_escape_string($string)
{

  	$string = str_replace("'", "\\'", $string);
  	$string = str_replace("\\n", "\\\\n", $string);
  	$string = str_replace("\\r", "\\\\r", $string);

	return $string;
}

/**
 * Parse global arrays, and strips them of naughty stuff
 *
 * @param array $global
 * @param bool $magicquotes
 * @param bool $html
 * @return array $global
 * @todo This should save already parsed arrays so avoid parsing them several times in a page load
 */
function parse_global($global, $magicquotes, $html)
{
	$return = array();

	foreach($global as $key => $value)
	{
		$key   = sql_escape_string($key);
		$value = sql_escape_string($value);

		if ($magicquotes == true)
		{
			$value = strip_magic_quotes_gpc($value);
		}
		if (($html == true) && (!is_array($value)))
		{
			$value = htmlentities($value);
		}

		$key   = trim($key);

		//let's recurse!
		if (is_array($value))
		{
			$return[$key] = parse_global($value, $magicquotes, $html);
		}
		else
		{
			// Check for the dangerous Poison NULL Byte attack
			if (strchr($value, null))
			{
				trigger_error('Security: Poison NULL Byte attack attempted', E_USER_WARNING);
				unset($value);
			}
			else
			{
				$return[$key] = $value;
			}
		}
	}
	return $return;
}

/**
 * Returns the clock in unix timestamp, is to be improved
 *
 * @param int $timestamp
 * @return int $time
 * @todo remove global $config class.
 */
function forumtime($timestamp = false)
{
	global $config;

	//Either use timestamp or unix time
	if ($timestamp)
	{
		$time = $timestamp;
	}
	elseif ($timestamp == false)
	{
		$time = time();
	}

	//Adjust it
	$adjust = ifsetor($config->time_adjust, 0);

	if (stristr($adjust, '-') === false)
	{
		return ($time + $adjust);
	}
	else
	{
		$adjust = str_replace('-', '', $adjust);
		return ($time - $adjust);
	}
}

/**
 * Output a error
 *
 * @param string $msg
 * @param bool $add_wrapper
 */
function error_msg($msg, $add_wrapper = true)
{
	//uck, globalise
	global $data, $config, $screen, $user;

	//If something is really fucked up, we are only going to show the error and cry
	if(!is_object($data) || !is_object($screen) || !$user || !is_object($config))
	{
		trigger_error('PHP: Fatal error! Data, Screen, User or Config is missing
						while attempting to throw this error: ' . $msg, E_USER_ERROR);
		die("<b>Vikingboard fatal error</b><br />\n{$msg}");
 	}

	//Set language
	if (defined('IN_ACP'))
	{
		$location = 'admin';
	}
	else
	{
		$location = 'index';
	}

	if (!(include_once('inc/lang/'.$config->def_lang.'/' . $location . '.php')))
	{
		include_once('inc/lang/1/' . $location . '.php');
	}
	$lang_global    =    new lang_global;

	//we are not in ACP
	if (!defined('IN_ACP'))
	{
		//add template
		$tohtml = array('message' => $msg,
					'imgset' => $user->skin_set);

		$screen->add_html('message', $tohtml);
	}

	//wrap? on or out of ACP?
	if ($add_wrapper && !defined('IN_ACP'))
	{
		$navigation['index.php']	=	$lang_global->info;
		$screen->add_wrapper($lang_global->info, $navigation);
	}
	elseif ($add_wrapper && defined('IN_ACP'))
	{
		$html = $screen->return_html('message', array('message' => $msg, 'imgset' => $user->skin_set));
		$screen->add_admin_wrapper($html);
	}
	//lets print it
	$screen->print_page();

	//and stop the script
	exit();
}

/**
 * Output a notification with a link.
 *
 * @param string $header Notification header. Is also placed in <title>-tag and secondary navigation
 * @param string $message The notification message
 * @param string $target The target of the link (address)
 * @param string $link_text Link text
 */
function notify($header, $message, $target = false, $link_text = false)
{
	global $config, $screen, $user, $session;

	$lang	=	new lang_global;

	// If no target URL is provided, check if the user came from ... somewhere
	if (!$target && $session->get('reffering_page') != '')
	{
		$target	=	$session->get('reffering_page');
	}

	// Add default back-text if nothing else was given
	if (!$link_text)
	{
		$link_text	=	$lang->back;
	}

	$html	=	array
	(
		'header'	=>	$header,
		'message'	=>	$message,
		'target'	=>	$target,
		'link'		=>	$link_text,
	);

	if (!defined('IN_ACP'))
	{
		$navigation[]	=	$header;
		$screen->add_html('notification', $html);
		$screen->add_wrapper($header, $navigation);
	}
	else
	{
		$html	=	$screen->return_html('admin_notification', $html);

		$screen->add_admin_wrapper($html);
	}

	$screen->print_page();

	// And stop it
	exit();
}

/**
 * Does the user really want this? Let's ask him/her.
 *
 * @param string $msg The message outputted to the user
 * @param array $targets An array of the available options
 */
function confirm($header, $message, $targets)
{
	global $user, $screen;

	//language stuff
	$lang_global = new lang_global;

	//loop the links
	$links	=	'';
	foreach ($targets as $text => $target)
	{
		$links	.=	$screen->return_html('basic_link', array('target' => $target, 'text' => $text));
	}

	//template stuff
	$html	=	array
	(
		'header'	=>	$header,
		'message'	=>	$message,
		'targets'	=>	$links,
	);

	if (!defined('IN_ACP'))
	{
		$screen->add_html('confirm', $html);

		//wrap it
		$navigation['index.php']	=	$lang_global->confirm;
		$screen->add_wrapper($lang_global->confirm, $navigation);

		//lets print it
		$screen->print_page();

		//and stop the script, waiting for the user to take action
		exit();
	}
	else
	{
		return $screen->return_html('confirm', $html);
	}

}

/**
 * Verifies that a string actually is an e-mail
 *
 * @author Cal Henderson {@link http://www.iamcal.com/publish/articles/php/parsing_email/},
 *	modified by ph to save already checked emails.
 * @license Creative Commons Attribution-ShareAlike 2.5 {@link http://creativecommons.org/licenses/by-sa/2.5/}
 * @param string $email
 * @return bool
 */
function is_valid_email_address($email)
{
	static $evaluated_emails	=	array();

	if( !isset( $evaluated_emails[$email] ) )
	{
		$qtext			=	'[^\\x0d\\x22\\x5c\\x80-\\xff]';
		$dtext			=	'[^\\x0d\\x5b-\\x5d\\x80-\\xff]';
		$atom			=	'[^\\x00-\\x20\\x22\\x28\\x29\\x2c\\x2e\\x3a-\\x3c'.
							'\\x3e\\x40\\x5b-\\x5d\\x7f-\\xff]+';
		$quoted_pair	=	'\\x5c[\\x00-\\x7f]';
		$domain_literal	=	"\\x5b($dtext|$quoted_pair)*\\x5d";
		$quoted_string	=	"\\x22($qtext|$quoted_pair)*\\x22";
		$domain_ref		=	$atom;
		$sub_domain		=	"($domain_ref|$domain_literal)";
		$word			=	"($atom|$quoted_string)";
		$domain			=	"$sub_domain(\\x2e$sub_domain)*";
		$local_part		=	"$word(\\x2e$word)*";
		$addr_spec		=	"$local_part\\x40$domain";

		$evaluated_emails[$email]	=	preg_match( "!^$addr_spec$!", $email );
	}

	return $evaluated_emails[$email];
}

/**
 * Verifies that the URL is valid
 *
 * @todo Actual validation (constructing GET requests and listen for replies)
 * @param string $url
 * @return bool
 */
function is_valid_url($url)
{
	static $evaluated_urls = array();

	if (!isset($evaluated_urls[$url]))
	{
		$evaluated_urls[$url] = false;

		// IP-based (numeric) URL
		if (preg_match('#(|(http|https)://)\b\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}\b#is', $url, $matches))
		{
			/* We assume that if the URL is well-formed for the entire hostname,
			 it's going to be well-formed for the rest of the URL */
			if ($matches[0])
			{
				$evaluated_urls[$url] = true;
			}
		}

		// Local URL
		if (strchr($url, 'localhost'))
		{
			$evaluated_urls[$url] = true;
		}

		// Normal, non-local, hostname-based URL (thanks to marek_mar on codingforums.com <3)
		if (preg_match('#(?>[a-z+]{2,}://|www\.)(?:[a-z0-9]+(?:\.[a-z0-9]+)?@)?(?:(?:[a-z](?:[a-z0-9]|(?<!-)-)*[a-z0-9])(?:\.[a-z](?:[a-z0-9]|(?<!-)-)*[a-z0-9])+|(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?))(?:/[^\\/:?*"<>|\n]*[a-z0-9])*/?(?:\?[a-z0-9_.%]+(?:=[a-z0-9_.%:/+-]*)?(?:&[a-z0-9_.%]+(?:=[a-z0-9_.%:/+-]*)?)*)?(?:\#[a-z0-9_%.]+)?#i', $url, $matches))
		{
			$evaluated_urls[$url] = true;
		}
	}

	return $evaluated_urls[$url];
}

/**
 * Strip magic quotes
 *
 * Removes horrible magic quotes from $input, even if $input is a
 * multidimensional array
 *
 * @param string/array $input
 * @return string/array
 */
function strip_magic_quotes_gpc ($input)
{
	if (is_array($input) === true)
	{
		foreach ($input as $k => $v)
		{
			if (is_array($v) === true)
			{
				$input[$k] = strip_magic_qoutes_gpc($v);
			}
			else
			{
				$input[$k] = stripslashes($v);
			}
		}
	}
	else
	{
		$input = stripslashes($input);
	}

	return $input;
}

/**
 * Check password strength
 *
 * Checks for the password strength,
 * and rates it from 1-4 (higher is better)
 *
 * @author John Tindell
 * @param string $password
 * @return int $strength
 */
function password_strength($password)
{
	$strength = 0;
	$patterns = array('/[a-z]/', '/[A-Z]/', '/[0-9]/', '/[¬!"£$%^&*()`{}\[\]:@~;\'#<>?,.\/\\-=_+\|]/');

	foreach ($patterns as $pattern)
	{
		if (preg_match($pattern, $password))
		{
			$strength++;
		}
	}

	return $strength;
}

/**
 * Redirect
 *
 * Redirects the user to a specified URL, by whatever means necessary.
 * If no URL is specified, the user is redirected to the forum main
 *
 * @todo Add Admin CP switchability, like redirecting via META refreshes
 * for browsers or servers without support for Location: headers
 * @param string $url
 */
function redirect($url = '')
{
	global $config;

	header('Location: ' . $config->forum_url . $url);
	exit();
}

/**
 * Encrypt password
 *
 * Encrypts the password, utilizing static seeds and other methods.
 * MD5 forms the base encryption method.
 *
 * @todo Up the security a notch by using random seeds or per-user seeds.
 * 			- Rename to pwd_encrypt() to remove ambiguity
 *
 * @param string $password Password to encrypt
 * @return string Encrypted password
 */
function encrypt($password)
{
	// Get seed
	global $config;
	$seed = $config->seed;

	// Encrypt
	$enc_password = md5($seed . $password . md5('lol, 1337'));
	return $enc_password;
}

/**
 * ifsetor()
 *
 * If variable $param is set, return it. If not, return $or. Useful for globals,
 * where you most often won't know whether they are set or not, to avoid getting
 * the nasty notices.
 *
 * The PHP internals debated whether or not to include this function in PHP6, but
 * ultimately voted no.
 *
 * @param string $param Variable to check
 * @param string $or Value to assign to it if not set
 * @return mixed Either $param or $or
 */
function ifsetor(&$param, $or)
{
	return isset($param) ? $param : $or;
}

/**
 * Return a safe integer, or default value
 * if integer is 0.
 *
 * @param mixed $var
 * @param mixed $default
 * @return int
 */
function safe_int($var, $default)
{
	$var = intval($var);

	return ($var == 0) ? $default : $var;
}

/**
 * Returns a safe integer, or default if integer
 * is zero or negative
 *
 * @param mixed $var
 * @param mixed $default
 * @return int $var or mixed $default
 */
function safe_pos_int($var, $default)
{
	$var = intval($var);
	return ($var <= 0) ? $default : $var;
}

/**
 * Get a user supplied integer, and safen it up.
 *
 * Fetches the integer from GET or POST (in that order), and
 * checks that it's safe (safe_int()), and that it is > 0.
 *
 * @param mixed $var
 * @param mixed $default
 * @return int $var or mixed $default
 */
function safe_user_int($var, $default)
{
	$return = ifsetor($_GET[$var], $default);
	$return = safe_int($return, $default);

	if ($return == $default)
	{
		$return = ifsetor($_POST[$var], $default);
		$return = safe_int($return, $default);
	}

	return ($return < 0) ? $default : $return;
}

/**
 * Gets hold of remote clients IP-address
 *
 * @return String $ip
 */
function get_remote_address()
{
	$ip = $_SERVER['REMOTE_ADDR'];

	/**
	 * @todo 	Support for proxys, e.g ApacheX`s mod_proxy ($_SERVER['HTTP_X_FORWARDED_FOR'],
	 * 			but that can easily be forged.
	 */

	//Set IP to unkown if its..well, unknown
	if (strlen($ip) < 7)
	{
		$ip = 'unknown';
	}

	return $ip;
}

/**
 * Truncates a word to given length
 *
 * @param string string Text to be truncated
 * @param int max_length Maximum length of string after truncation
 * @param int start_truncate Where in the string the seperator should start
 * @param string seperator Marks where string has been truncated
 * @return string
 */
function truncate_word($string, $max_length = 50, $start_truncate = 35, $seperator = '...')
{
	if ((strlen($string) > $max_length) && ($max_length > $start_truncate))
	{
		$string	=	substr_replace($string, $seperator, $start_truncate, ($start_truncate + strlen($seperator) - $max_length));
	}
	return $string;
}

/**
 * Human time
 *
 * Calculates the number of {periods} since timestamp
 * @example echo human_time(time() - 3780);
 *		outputs '1 hour ago'
 * @return string text '{num} {periods} ago'
 * @todo Add text strings to language file. Add support for timestamps > time() ?
 */
function human_time($timestamp)
{
	$diff	=	time() - $timestamp;

	$periods	=	array
	(
		array('%d second ago', '%d seconds ago', 1),
		array('%d minute ago', '%d minutes ago', 60),
		array('%d hour ago', '%d hours ago', 3600),
		array('%d day ago', '%d days ago', 86400),
		array('%d week ago', '%d weeks ago', 604800),
		array('%d month ago', '%d months ago', 2629744),
		array('%d year ago', '%d years ago', 31556926),
	);

	if ($diff == 0)
	{
		$number	=	$i	=	0;
	}
	else
	{
		$num_periods	=	count($periods);
		for ($i = $num_periods - 1; ($i >= 0) && (($number = $diff / $periods[$i][2]) < 1); $i--);
		$number	=	floor($number);
	}

	return sprintf($periods[$i][(($number != 1) ? 1 : 0)], $number);
}

/**
 * Human time (new implementation)
 *
 * @todo To be written in 0.2.1/0.3/0.4
 *
 * @param int $timestamp1
 * @param int $timestamp2
 * @return string Time ago/until
 *//*
function human_time($timestamp1, $timestamp2)
{

}*/

/**
 * Date dropdown
 *
 * Returns the HTML of a dropdown-menu to select a date. Default selected values
 * may be supplied, otherwise the default will be 1/1-2000
 *
 * @todo Perhaps use the current date rather than 1-1-2000 as default value?
 *
 * @param Screen $screen
 * @param int $day
 * @param int $month
 * @param int $year
 * @return string HTML output
 */
function date_dropdown(&$screen, $day = 01, $month = 01, $year = 2000)
{
	$lang = new lang_time();

	// Days
	$days = '';
	for ($i = 1; $i < 32; ++$i)
	{
		if ($i == $day)
		{
			$selected = 'selected="selected" ';
		}
		else
		{
			$selected = '';
		}

		$days .= $screen->return_html('dropdown_option', array('value' => $i, 'selected' => $selected, 'name' => $i));
	}

	// Month
	$months = '';
	for ($i = 1; $i < 12; ++$i)
	{
		if ($i == $month)
		{
			$selected = 'selected="selected" ';
		}
		else
		{
			$selected = '';
		}

		$lang_month = 'month_' . $i;

		$months .= $screen->return_html('dropdown_option', array('value' => $i, 'selected' => $selected, 'name' => $lang->$lang_month));
	}

	// The rest
	return $screen->return_html('date_dropdown', array('days' => $days, 'months' => $months, 'year' => $year));
}

/**
 * Get user accepted languages
 *
 * @return Array $user_languages Array of user accepted languages, ordered by acceptability
 * from 1.0 and down.
 */
function user_languages()
{
	$languages = explode(',', $_SERVER['HTTP_ACCEPT_LANGUAGE']);

	foreach ($languages as $language)
	{
		$language = explode(';', $language);

		if (!empty($language[1]))
		{
			preg_match('#q=([0-9]\.[0-9]+?)#is', $language[1], $q);
			$q = $q[1];
		}
		else
		{
			$q = "1.0";
		}

		$user_languages[$q] = $language[0];
	}

	krsort($user_languages, SORT_NUMERIC);
	return $user_languages;

}

?>