<?php
/**
 *
 * @name 		Parser
 *
 * @desc		This file is the post parser. It handles posts or other
 * 				strings, and parses/deparses the BBCode in them, as well as
 * 				filtering out bad content (bad language or bad formatting).
 *
 * 				Please don't comment the fugly hacks in here, I know about them,
 * 				and I'm also terribly ashamed of them. Someone else should do the
 * 				next rewrite.
 *
 * @author 		Jørgen Tellnes <jorgis@jorgis.com>
 * @version 	0.2
 *
 * @package 	Vikingboard
 * @copyright 	Copyright (c) 2004-2007 The Vikingboard Project
 * @license 	GPL {@link http://www.gnu.org/licenses/gpl.html}
 * @link 		http://vikingboard.com/
 *
 * @example  	include parser.php;
 * 				$Parser 	= new Parser;
 * 				$parsed_str = $Parser->parse($raw_str);
 *
 * $Id: parser.php 1885 2007-12-03 00:16:43Z jorgenboard $
 */

//oh no!
if (!defined('VBOARD'))
{
	die('Hacker attempt in ' . basename(__FILE__));
}


/**
 * Post Parser
 *
 */
class Parser
{

	/**
	 * Main constructor method, references the supplied vars within the class.
	 *
	 * @param Data $data
	 * @param Screen $screen
	 * @param Config $config
	 * @param int $parse_smileys = 1
	 * @return Constructor
	 */
	function Parser($data, $screen, $config, $parse_smileys = 1)
	{
		$this->data 	= $data;
		$this->screen 	= $screen;
		$this->config	= $config;

		if (!defined('IN_ACP'))
		{
			$this->lang		= new lang_bbcode;
		}

		// Settings
		$this->parse_smileys = $parse_smileys;

	}

	/**
	 * Parse before storing
	 *
	 * @access Public
	 * @param string $in
	 * @return mixed $in on success, array $failure on failure. $failure[0] = error_info, $failure[1] = unparsed post
	 */
	function parse($input)
	{
		$in = $input;

        // Strip escaping
        $in = strip_magic_quotes_gpc($in);

		// Badwords
		$badwords = $this->data->cachedQuery('get_badwords', array());

		if (is_array($badwords))
		{
			foreach ($badwords as $badword)
			{
				// Translating *-style wildcards into PCRE-style regexps
				$regex = str_replace('*', '([^\s\W]+?)', $badword->name);

				// Doing the regex-stuff itself
				$in = preg_replace("/\b$regex\b/i", $badword->nname, $in);
			}
		}

		// Making stuff somewhat safe
		$in = htmlentities($in, null, 'UTF-8');
		$in	= nl2br($in);

		// Smilies
		if ($this->parse_smileys)
		{
			$smileys = $this->data->cachedQuery('get_smileys');

			$num_smileys = 0;

			foreach ($smileys as $smile)
			{
				$smilecode = $this->screen->return_html('smiley', array('code' 		=> $smile->code,
																		'set_id' 	=> $smile->set_id,
																		'filename' 	=> $smile->url,
																		'name' 		=> $smile->name));
				// Find number of smileys
				$num_smileys += substr_count($in, $smile->code);

				$in	= str_replace($smile->code, $smilecode, $in);
			}

			// If more than allowed, fail it and return the amount (for the error msg)
			if ($num_smileys > $this->config->max_smileys)
			{
				return array($num_smileys, $input);
			}
		}

		// Get all BBCode parse rules
		$parse_rules = $this->get_parse_rules();

		// Construct the replace and find regexes
		foreach ($parse_rules as $find => $replace)
		{
			// Locate all character classes
			preg_match_all('#:.+?:#is', $find, $matches);

			// Escape all symbols that may mess up the regex
			$find = addcslashes($find, '#[]\.<>?+$');

			$find_regex[] = '#' . $find . '#is';

			// Add regex variables in the replace regexes
			foreach ($matches as $match)
			{
				// An ugly hack, but it saves me a lot of foreach loops and array regex stuff.
				$replace_regex[] = str_replace($match, array('$1', '$2', '$4', '$5', '$6', '$7', '$8', '$9', '$10'), $replace);
			}
		}

		// Character classes
		$simple_classes = array(':text:', ':url:', ':number:', ':word:', ':date:');
		$regex_classes	= array('(|.+?)', '((http://|www)[^\'"\s]+?)', '([0-9]+?)', '([^"\s]+?)', '(lol)');

		$find_regex = str_replace($simple_classes, $regex_classes, $find_regex);

		// Handle the special case of [quote=user:*id*]
		if (preg_match('#\[quote=post\:([0-9]+?)\]#is', $in, $postid))
		{
			$postid = $postid[1];

			$post = $this->data->cachedQuery('get_post_info', array($postid));
			$post = $post[0];

			$cite = str_replace('%uid%', $post->author_id, $this->lang->cite);
			$cite = str_replace('%username%', $post->author_name, $cite);
			$cite = str_replace('%posted%', date($this->config->timestamp1, $post->posted), $cite);

			$in = preg_replace('#\[quote=post\:([0-9]+?)\]#is', $cite, $in);
		}

		// Apply all regexes to $in
		$in = preg_replace($find_regex, $replace_regex, $in);

		// Handle the special case of messed UP urls
		$in = str_replace('http://http://', 'http://', $in);

		// Handle the special cases of [PHP], [HTML] and [CSS]. This is by far the dirties hack I've EVER seen. Please go away, to avoid damage to eyes.
		preg_match('#\[(php|css|html)\](|.+?)\[\/(php|css|html)\]#is', $in, $matches);

		if ($matches)
		{
			$func_name = 'highlight_' . $matches[1];
			$highlight_code = '<pre><code class="' . $matches[1] . '">' . $this->$func_name(html_entity_decode($this->br2nl($matches[2]), null, 'UTF-8')) . '</code></pre>';
			$in = preg_replace('#\[(php|css|html)\](|.+?)\[\/(php|css|html)\]#is', $highlight_code, $in);
		}

		// Then we return to bissniss
		return $in;
	}

	/**
	 * Deparse before edit
	 *
	 * @access Public
	 * @param string $in Parsed content, to deparse
	 * @param bool $no_formatting Returns post without formatting (neither BBCode nor HTML)
	 * @return string Deparsed content
	 */
	function deparse($in, $no_formatting = false)
	{
		// Output without any formatting
		if ($no_formatting == true)
		{
			$in = strip_tags($in);

			return $in;
		}

		// Deparse the smileys :D
		$smileys = $this->data->cachedQuery('get_smileys');

		foreach ($smileys as $smile)
		{
			$smilecode = $this->screen->return_html('smiley', array('code' 		=> $smile->code,
																	'set_id' 	=> $smile->set_id,
																	'filename' 	=> $smile->url,
																	'name' 		=> $smile->name));

			$in	= str_replace($smilecode, $smile->code, $in);
		}

		// Get BBCode rules (backwards)
		$parse_rules = array_flip($this->get_parse_rules());

		// Transform to reversed regexes
		foreach ($parse_rules as $find => $replace)
		{
			// All char classes
			preg_match_all('#:.+?:#is', $find, $matches);

			// Escaping bad stuff
			$find = addcslashes($find, '#[]\.?+');

			$find_regex[] = '#' . $find . '#is';

			// Add regex variables in the replace regexes
			foreach ($matches as $match)
			{
				// An ugly hack, wuah
				$replace_regex[] = str_replace($match, array('$1', '$2', '$4', '$5', '$6', '$7', '$8', '$9', '$10'), $replace);
			}
		}

		// Character classes
		$simple_classes = array(':text:', ':url:', ':number:', ':word:', ':date:');
		$regex_classes	= array('(|.+?)', '([^\'"\s]+?)', '([0-9]+?)', '([^"\s]+?)', '(lol)');

		$find_regex = str_replace($simple_classes, $regex_classes, $find_regex);

		// Apply all regexes to $in
		$in = preg_replace($find_regex, $replace_regex, $in);

		// The special cases of [php], [html] and [css]. Turn away now, or your eyes WILL bleed.
		preg_match('#<pre><code class="([^"\s]+?)">(.+?)</code></pre>#is', $in, $matches);
        if(count($matches) > 2) {
		    $replace_with = "[{$matches[1]}]" . $this->br2nl(strip_tags(str_replace("\n", '', $matches[2]), '<br><br />'), true) . "[/{$matches[1]}]";
		    $in = preg_replace('#<pre><code class="([^"\s]+?)">(.+?)</code></pre>#is', $replace_with, $in);
        }

		// Clean up
		$in = $this->br2nl($in);
		$in = strip_tags($in);

		// And return to base
		return $in;
	}


	// Default highlight methods

	/**
	 *
	 * @access Private
	 * @param string $code
	 * @return Done
	 */
	function highlight_html($code)
	{
		/**
		* @todo add more tags, ex. all form tags, including textfields
		*/
		$code = htmlentities($code);

		// strings
		$code = preg_replace('#&quot;(.+?)&quot;#is', '<span style="color:#0000FF;">&quot;$1&quot;</span>', $code);

		// highlighting of generic tags
		$code = preg_replace("#&lt;([^!]+?)&gt;#is", "<span style=\"color:#000099;\">&lt;\\1&gt;</span>", $code);

		// <a>
		$code = preg_replace("#&lt;a(.+?)&gt;(.+?)&lt;/a&gt;#is", "<span style=\"color:#006600;\">&lt;a\\1&gt;</span>\\2<span style=\"color:#006600;\">&lt;/a&gt;</span>", $code);

		// <img>
		$code = preg_replace("#&lt;img(.+?)&gt;#is", "<span style=\"color:#990099;\">&lt;img\\1&gt;</span>", $code);

		// <input>
		$code = preg_replace("#&lt;input(.+?)&gt;#is", "<span style=\"color:#FF9900;\">&lt;input\\1&gt;</span>", $code);

		// <style>
		$code = preg_replace("#&lt;style&gt;(.+?)&lt;/style&gt;#is", "<span style=\"color:#990099;\">&lt;style&gt;</span>\\1<span style=\"color:#990099\">&lt;/style&gt;</span>", $code);

		// <!-- and -->
		$code = preg_replace("#&lt;!--(.+?)--&gt;#is", "<span style=\"color:#999999;\">&lt;!--\\1--&gt;</span>", $code);

		// <script> with closing tag
		$code = preg_replace("#&lt;script(.+?)&gt;(.+?)&lt;/script&gt;#is", "<span style=\"color:#990000;\">&lt;script\\1&gt;</span>\\2<span style=\"color:#990000;\">&lt;/script&gt</span>", $code);

		// <script> w/o closing tag
		$code = preg_replace("#&lt;script(.+?)&gt;#is", "<span style=\"color:#990000;\">&lt;script\\1&gt;</span>", $code);

		// <form>
		$code = preg_replace("#&lt;form(.+?)&gt;#is", "<span style=\"color:#FF9900;\">&lt;form\\1&gt;</span>", $code);

		// </form>
		$code = preg_replace("#&lt;/form(.+?)&gt;#is", "<span style=\"color:#FF9900;\">&lt;/form\\1&gt;</span>", $code);

		//indeed.
		return $code;
	}

	/**
	 *
	 * @access Private
	 * @param string $code
	 * @return string Highlighted PHP code
	 */
	function highlight_php($code)
	{
		return highlight_string($code, true);
	}

	function highlight_css($code)
	{
		$code = htmlentities($code);
		return $code;
	}

	function highlight_sql()
	{

	}

	/**
	 * Get parse rules
	 *
	 * Get all the BBCode/other parse rules (including custom ones) from DB,
	 * and transform into an associative array
	 *
	 * @access Private
	 * @return array $rules
	 */
	function get_parse_rules()
	{
		$rules_db = $this->data->cachedQuery('get_parse_rules');

		$rules = array();

		foreach ($rules_db as $r)
		{
			$rules[$r->find] = $r->replace;
		}

		return $rules;
	}

	/**
	 * @abstract Parse rules:
	 *
	 *	Replace:
	 *		An expression containing variables, designed to match the tag you wish to add support for. The match
	 *		is case-insensitive.
	 *
	 *	Variables:
	 *		:text: 		- Ordinary text, including linebreaks
	 *		:url: 		- An URL, with or without "www". May also include mail-urls. No whitespace allowed.
	 *		:number:	- A number
	 *		:word:		- A short text, like a name, CSS class or similar. No whitespace.
	 *		:date:		- A date and/or time
	 *
	 *		Note that you only can use a variable once in your replace expression. You shouldn't need to either.
	 *
	 *	Functions:
	 *		PHP[function(variable)] - Runs the function "function" with "variable" as parameters. The variables from
	 *								  the replace-part may be used, as in the examples. Only some functions will be permitted.
	 *
	 *	Examples:
	 *		PHP code highlighting:
	 *		[php]:text:[/php] => <code class="php-code">PHP[highlight_php(:text:)]</code>
	 *
	 *		You could make an extended [img] tag:
	 *		[img=:url:]:text:[/img] => <div class="imagebox"><img src=":url:"> alt=":text:" /><p class="caption">:text:</p></div>
	 */

	/**
	 * Is allowed function
	 *
	 * What functions are allowed to use in BBCodes? We wouldn't want a cracker to use
	 * system(), exec() or worse, do we?
	 *
	 * @todo Needs extensive expansion of the whitelist
	 *
	 * @param string $function
	 * @return bool
	 */
	function is_allowed_function($function)
	{
		$class_methods = get_class_methods($this);
		$forbidden_methods = array('Parser', 'parse', 'deparse', 'get_parse_rules', 'is_allowed_function', 'br2nl');

		$allowed = array('ucfirst', 'strtoupper', 'strtolower', 'ucwords');

		if (in_array($function, $class_methods) && !in_array($function, $forbidden_methods))
		{
			return '$this->' . $function;
		}
		elseif (in_array($function, $allowed))
		{
			return $function;
		}
		else
		{
			return false;
		}
	}

	/**
	 * <br /> to newline
	 *
	 * The exact opposite of nl2br()
	 *
	 * @param string $in
	 * @return string $out
	 */
	function br2nl($in, $add_newlines = false)
	{
		if ($add_newlines == true)
		{
			$replace_with = "\n";
		}
		else
		{
			$replace_with = '';
		}

		// XHTML-style
		$in = str_ireplace('<br />', $replace_with, $in);

		// HTML 4.01-style (PHP < 4.0.5)
		$in = str_ireplace('<br>', $replace_with, $in);

		return $in;
	}
}

?>